/*
 Cmd.cc - Cmd class implementation

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/

#include "Cmd.h"
#include "CmdList.h"
#include <X11/Intrinsic.h>

extern Cmd *backtrack;	// extern object reversing most recent command
Cmd *Cmd::_lastCmd = NULL;	// ptr to most recent Cmd


Cmd::Cmd ( const char *name, int active )
{
	_name = XtNewString ( name );
	_active = active;
	_hasUndo = TRUE;	// derived classes are expected to implement undo()
	_activationList = NULL;
	_deactivationList = NULL;
	_undoList = NULL;
}


Cmd::~Cmd()
{
	XtFree ( _name );
	delete _activationList;
	delete _deactivationList;
	delete _undoList;
}


void Cmd::registerInterface ( CmdInterface *ci )
{
	_ci.Add ( ci );	

	if ( ci )
		if ( _active )
			ci->activate();
		else
			ci->deactivate();
}


void Cmd::activate()
{
	for ( int i = 0; i < _ci.Count(); i++ )
		_ci[i]->activate();
		
	// save current value of active before setting the new state
	_previouslyActive = _active;
	_active = TRUE;
}


void Cmd::deactivate()
{
	for ( int i = 0; i < _ci.Count(); i++ )
		_ci[i]->deactivate();
		
	_previouslyActive = _active;
	_active = FALSE;
}


void Cmd::revert()
{
	if ( _previouslyActive )
		activate();
	else
		deactivate();
}


void Cmd::addToActivationList ( Cmd *cmd )
{
	if ( !_activationList )
		_activationList = new CmdList();

	_activationList->add ( cmd );
}

void Cmd::addToDeactivationList ( Cmd *cmd )
{
	if ( !_deactivationList ) 
		_deactivationList = new CmdList();
		
	_deactivationList->add ( cmd );
}


void Cmd::addToUndoList ( Cmd *cmd )
{
	if ( !_undoList ) 
		_undoList = new CmdList();
		
	_undoList->add ( cmd );
}


void Cmd::removeFromUndoList ( Cmd *cmd )
{
	if ( _undoList )
		_undoList->remove ( cmd );
}


void Cmd::execute()
{
	if ( !_active )
		return;
		
	doCommand();	// call derived class' function to perform obj. action	
	
	if ( _hasUndo )
	{	
		Cmd::_lastCmd = this;
		backtrack->activate();
	}
	else
	{
		Cmd::_lastCmd = NULL;
		backtrack->deactivate();
	}

	// process Cmds dependent on 'this' one
	if ( _activationList )
		_activationList->activate();
		
	if ( _deactivationList )
		_deactivationList->deactivate();
}


void Cmd::undo()
{
	undoCommand();	// call the class' undoCommand() member function
	backtrack->deactivate();	// only 1 level undo implemented at present

	if ( _activationList )
		_activationList->revert();
		
	if ( _deactivationList )
		_deactivationList->revert();
}

/*
  Notes:
  
  Classes derived from Cmd must implement the pure virtual doCommand() and
  undoCommand() member functions.
  
  The access function returning the type of a Cmd object supports various
  uses of the CmdInterface class, e.g. menus and toolbars.
  
  The default type of Cmd is Action; derived classes can override this, e.g
  with Toggle; when an interface is associated with such a Cmd, the
  XmToggleButton widget might be used.
  
  The Cmd class supports a public method registerInterface(), registering a
  user interface component with a Cmd object; each Cmd object can be
  associated with more than one user interface component. 
  
  Once a CmdInterface object is registered with a Cmd object, the latter
  can change the sensitivity of the interface depending on whether the Cmd
  is active. As each CmdInterface component is registered,
  registerInterface() updates the component according to the current active state
  of the Cmd.
  
  activate() enables a Cmd object; if there are interface components
  asscoiated with this Cmd, this function calls each CmdInterface
  object's activate() member function.

  The Cmd class supports two (three) CmdList objects; 1) a list of Cmd
  objects for activation when a Cmd is executed; 2) a list of Cmd objects
  to be deactivated on execution of a Cmd; ( 3) a history list of Cmds to
  date for multiple undo ). 
*/
